#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#define	F_CPU				(16000000)		//  
#define	TIMER0_FREQ			(1000)			//    
#define	TIMER0_PRESCALER	(64)			//  

//  ,         TIMER0_FREQ
#define	TIMER0_RELOAD_LO	((0xFFFF-(F_CPU/TIMER0_PRESCALER/TIMER0_FREQ)+1) & 0xFF)
#define	TIMER0_RELOAD_HI	((0xFFFF-(F_CPU/TIMER0_PRESCALER/TIMER0_FREQ)+1) >> 8)

// ,  8-  .
#if (TIMER0_RELOAD_HI!=0xFF)
#error TIMER0_RELOAD_HI!=0xFF!
#endif

//      
// ( ,    .  100)
#define DEBOUNCE_CNT	(100)

// ByteBits:    .
//      ,     .
//  ,   flags  ByteBits:
//  -  :	flags.byte,
//  -  0:		flags.bits.FL_JB_1
//  -  1:		flags.bits.FL_OK_1
//    ,    .
typedef union
{
    unsigned char	byte;
    struct
    {
	    unsigned char	FL_JB_1 :1;		//  0
    	unsigned char	FL_OK_1 :1;		//  1
	}bits;
}ByteBits;

//  
static ByteBits flags = {0x20};

// 
static unsigned char counter;

//    0
SIGNAL(SIG_OVERFLOW0)
{
	unsigned char ch;

	TCNT0 = TIMER0_RELOAD_LO;	//  

	ch = PINB & 0x01;			//  
	ch ^= flags.byte;			// xor  

	//  .
	// "00 -  RETI (    )"

	// "01 -  FL_JB_1=0  RETI (  , )"
	if (ch == 1)
    {
		flags.bits.FL_JB_1 = 0;
    }

	// "10 -  FL_JB_1=1,    RETI"
    else if (ch == 2)
    {
		flags.bits.FL_JB_1 = 1;
		counter = 0;
    }

	// "11 -  +1"
    else if (ch == 3)
    {
		if (++counter == DEBOUNCE_CNT)	// !
		{
			if (flags.bits.FL_OK_1)	//   (     :-))
			{
				flags.bits.FL_OK_1 = 0;
			}
			else					//  
			{
				flags.bits.FL_OK_1 = 1;
			}
			flags.bits.FL_JB_1 = 0;
		}
    }

}



int main(void)
{
	// 
//	flags.byte = 0x20;

	TCCR0 = _BV(CS01) | _BV(CS00);	//	prescaler = 64.
  	TCNT0 = TIMER0_RELOAD_LO;		//
	TIMSK |= _BV(TOIE0);

	sei();		//  

	while (1);
}

